/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/

#include "mcu.h"
#include "base_types.h"

/*****************************************************************************
 *  DESCRIPTION:    Initialisation of the UART 
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        None
 *****************************************************************************/
void Uart_Init(void)
{
  /* SIN0_0 Pin */ 
  bFM0P_GPIO_PFR2_P21 = 1;
  FM0P_GPIO->EPFR07 &= ~(0x03 << 4);
  FM0P_GPIO->EPFR07 |= (0x01 << 4);
                
  /* SOT0_0 Pin */
  FM0P_GPIO->ADE &= ~(1 << 7);
  bFM0P_GPIO_PFR2_P22 = 1;
  FM0P_GPIO->EPFR07 &= ~(0x03 << 6);
  FM0P_GPIO->EPFR07 |= (0x01 << 6);
  
  FM0P_MFS0_UART->SCR  = 0x80;
  FM0P_MFS0_UART->SMR  = 0x01;
  FM0P_MFS0_UART->SSR  = 0x00;
  FM0P_MFS0_UART->ESCR = 0x00;
  FM0P_MFS0_UART->BGR  = 40000000 / 115200 - 1;  // 115.2kbaud @ 40 MHz
  FM0P_MFS0_UART->SCR  = 0x03;

}

/*****************************************************************************
 *  DESCRIPTION:    sends a single character 
 *
 *  PARAMETERS:     single character
 *
 *  RETURNS:        None
 *****************************************************************************/
void Uart_Putch(char_t c)
{

  while (bFM0P_MFS0_UART_SSR_TDRE == 0 );	// wait for transmit buffer empty
  FM0P_MFS0_UART->TDR = c;            /* put ch into buffer	*/
}

/*****************************************************************************
 *  DESCRIPTION:    sends a complete string (0-terminated) 
 *
 *  PARAMETERS:     Pointer to (constant) file of bytes in mem
 *
 *  RETURNS:        None
 *****************************************************************************/
void Uart_Puts(char_t *pu8Buffer)              
{
  while (*pu8Buffer != '\0')
  { 
    Uart_Putch(*pu8Buffer++);        // send every char of string
  }
}

/*****************************************************************************
 *  DESCRIPTION:    Waits for any input character 
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        Receive character (in case of error 0 will be returned)
 *****************************************************************************/
char_t Uart_Getch()
{

  if (bFM0P_MFS0_UART_SSR_RDRF == 0 )
  {
    return 0;
  }
  else
  if (FM0P_MFS0_UART->SSR & 0x38) 
  {	/* overrun or parity error */
        bFM0P_MFS0_UART_SSR_REC = 1;
        return 0;
  } 
  else
  {
    return (FM0P_MFS0_UART->RDR);
  }
}


/**
 ******************************************************************************
 ** \brief  Main function of project for S6E1A1 series.
 **
 ** \param  none
 ** \return uint32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
  uint8_t ch;

  bFM0P_GPIO_PDOR6_P61 = 1; // set LED = off
  bFM0P_GPIO_DDR6_P61 = 1;  // use LED -> output

  bFM0P_GPIO_DDR0_P04 = 0;  // key button SW3 'INT' -> Input

  Uart_Init();
  Uart_Puts("\r\nHello World ... Any key will be echoed ... Press any key now!\r\n");
  Uart_Puts("(Press 1 to switch on the user-LED, press 0 to switch it off)\r\n");
  Uart_Puts("(Press SW3(INT) to output a message)\r\n\r\n");

  while(1)
  {
    // UART handling
    
    ch = Uart_Getch(); // Read character from UART
    if (ch != 0)
    {
      Uart_Putch(ch);  // Echo character

      // Switch User-LED: ON by pressing '1', OFF by pressing '0'
			
      if (ch == 48)
      {
        bFM0P_GPIO_PDOR6_P61 = 1; // set LED = off
        Uart_Puts(" -> LED2(User) switched off!\r\n");
      }
      if (ch == 49)
      {
        bFM0P_GPIO_PDOR6_P61 = 0; // set LED = on
        Uart_Puts(" -> LED2(User) switched on!\r\n");
      }
    }
    
    // Check user key button SW3 (INT)
    
    if (bFM0P_GPIO_PDIR0_P04 == 0)
    {
      Uart_Puts("Key button SW3 pressed!\r\n");
      while (bFM0P_GPIO_PDIR0_P04 == 0);
    }
    
  }
}
